package jp.integ.struts.lessconfig.config.rule.impl;

import org.apache.struts.config.ModuleConfig;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;
import org.seasar.struts.lessconfig.config.rule.impl.DefaultActionPathNamingRule;

/**
 * パッケージ構成をパスにするアクションパスルールクラスです。
 * 
 * @author Jun Futagawa (Integsystem Corporation)
 * @version $Date:: 2006-09-20 18:43:25 +0900 $
 */
public class QualifiedActionPathNamingRule extends DefaultActionPathNamingRule {
	/** クラス名の大文字をパスに変換するかどうかを表します。 */
	private boolean splitUpperName = true;
	/** ルートパッケージ名を表します。 */
	protected String[] rootPackageNames;

	/**
	 * 大文字をパスに変換するかどうかを判定します。
	 * 
	 * @return 大文字をパスに変換するかどうか
	 */
	public boolean isSplitUpperName() {
		return splitUpperName;
	}

	/**
	 * 大文字をパスに変換するかどうかを設定します。
	 * 
	 * @param splitUpperName 大文字をパスに変換するかどうか
	 */
	public void setSplitUpperName(boolean splitUpperName) {
		this.splitUpperName = splitUpperName;
	}

	private S2Container getContainer() {
		return SingletonS2ContainerFactory.getContainer();
	}

	/**
	 * {@inheritDoc}
	 */
	public Class toComponentClass(ModuleConfig config, String path) {
		String prevPath;
		do {
			prevPath = path;
			path = toQualifiedComponentClass(path);
			Class clazz = super.toComponentClass(config, path);
			if (clazz != null) {
				return clazz;
			}
			path = toQualifiedNextPath(prevPath);
		} while (!prevPath.equals(path));
		return null;
	}

	/**
	 * 指定された文字列の / を _ に変換し、次の1文字を大文字にします。<br />
	 * from "/usr/Login" to "/user_Login"
	 * 
	 * @param path アクションクラスパス
	 * @return
	 */
	public static String toQualifiedComponentClass(String path) {
		path = path.replaceAll("//", "/");
		int length = path.length();
		if (length > 1 && path.charAt(length - 1) == '/') {
			path = path.substring(0, length - 1);
		}
		length = path.length();
		char chars[] = path.toCharArray();
		// 1文字目は無視
		for (int i = 1; i < length; i++) {
			if (chars[i] == '/' && length > i) {
				chars[i] = '_';
				chars[i + 1] = Character.toLowerCase(chars[i + 1]);
			}
		}
		return new String(chars);
	}

	/**
	 * 指定された文字列の最後の / を削除し、次の1文字を大文字にします。
	 * 
	 * @param path アクションクラスパス
	 * @return
	 */
	public static String toQualifiedNextPath(String path) {
		int index = path.lastIndexOf("/");
		int length = path.length();
		if (index > 0) {
			StringBuffer buffer = new StringBuffer();
			buffer.append(path.substring(0, index));
			if (length > index + 1)
				buffer.append(Character.toUpperCase(path.charAt(index + 1)));
			if (length > index + 2)
				buffer.append(path.substring(index + 2));
			return buffer.toString();
		}
		return path;
	}

	/**
	 * {@inheritDoc}
	 */
	public String toActionPathName(Class actionClass) {
		S2Container container = getContainer();
		if (!container.hasComponentDef(actionClass)) {
			return null;
		}
		if (rootPackageNames == null) {
			rootPackageNames = QualifiedRule.getRootPackageNames();
		}
		String actionPath = QualifiedRule.getActionPath(rootPackageNames,
				actionClass.getName(), isSplitUpperName());
		return actionPath;
	}
}
