/*
 * Copyright 2006 Integsystem Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package jp.integ.struts.lessconfig.config.rule.impl;

import junit.framework.TestCase;

/**
 * パッケージルートの処理ルールクラスのテストクラスです。
 * 
 * @author Jun Futagawa (Integsystem Corporation)
 * @version $Revision: 279 $ $Date: 2006-08-21 03:28:10 +0900 $
 */
public class QualifiedRuleTest extends TestCase {
	/**
	 * パッケージルートからルールに従いアクション定義パスを取得するテストです。
	 */
	public void testGetActionPath() {
		assertNull(QualifiedRule.getActionPath(null, null));
		assertEquals("/aaa", QualifiedRule.getActionPath(null, "aaaActionImpl"));
		assertEquals("/aaa", QualifiedRule.getActionPath(new String[0],
				"aaaActionImpl"));
		assertEquals("/aaa/bbb", QualifiedRule.getActionPath(new String[0],
				"aaa.bbbActionImpl"));
		assertEquals("/ccc", QualifiedRule.getActionPath(
				new String[] { "aaa" }, "aaa.cccActionImpl"));
		assertEquals("/bbb/ccc", QualifiedRule.getActionPath(
				new String[] { "aaa" }, "aaa.bbb.cccActionImpl"));
		assertEquals("/bbb/ccc", QualifiedRule.getActionPath(new String[] {
				"aaa", "bbb" }, "aaa.bbb.cccActionImpl"));
		assertEquals("/ccc/dddEee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb1.web", "aaa.bbb2.web" },
				"aaa.bbb1.web.ccc.DddEeeAction"));
		assertEquals("/ccc/dddEee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb2.web", "aaa.bbb1.web" },
				"aaa.bbb1.web.ccc.DddEeeAction"));
		assertEquals("/ccc/dddEee", QualifiedRule.getActionPath(new String[] {
				"aaa.web", "aaa.bbb1.web" }, "aaa.bbb1.web.ccc.DddEeeAction"));
		assertEquals("/ccc/dddEee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb1.web", "aaa.web" }, "aaa.bbb1.web.ccc.DddEeeAction"));
	}

	public void testGetActionPathWithNoSplitUpperName() {
		assertNull(QualifiedRule.getActionPath(null, null, false));
		assertEquals("/aaa", QualifiedRule.getActionPath(null, "aaaActionImpl",
				false));
		assertEquals("/aaa", QualifiedRule.getActionPath(new String[0],
				"aaaActionImpl", false));
		assertEquals("/aaa/bbb", QualifiedRule.getActionPath(new String[0],
				"aaa.bbbActionImpl", false));
		assertEquals("/ccc", QualifiedRule.getActionPath(
				new String[] { "aaa" }, "aaa.cccActionImpl", false));
		assertEquals("/bbb/ccc", QualifiedRule.getActionPath(
				new String[] { "aaa" }, "aaa.bbb.cccActionImpl", false));
		assertEquals("/bbb/ccc", QualifiedRule.getActionPath(new String[] {
				"aaa", "bbb" }, "aaa.bbb.cccActionImpl", false));
		assertEquals("/ccc/dddEee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb1.web", "aaa.bbb2.web" },
				"aaa.bbb1.web.ccc.DddEeeAction", false));
		assertEquals("/ccc/dddEee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb2.web", "aaa.bbb1.web" },
				"aaa.bbb1.web.ccc.DddEeeAction", false));
		assertEquals("/ccc/dddEee", QualifiedRule.getActionPath(new String[] {
				"aaa.web", "aaa.bbb1.web" }, "aaa.bbb1.web.ccc.DddEeeAction",
				false));
		assertEquals("/ccc/dddEee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb1.web", "aaa.web" }, "aaa.bbb1.web.ccc.DddEeeAction",
				false));
	}

	public void testGetActionPathWithSplitUpperName() {
		assertNull(QualifiedRule.getActionPath(null, null, true));
		assertEquals("/aaa", QualifiedRule.getActionPath(null, "aaaActionImpl",
				true));
		assertEquals("/aaa", QualifiedRule.getActionPath(new String[0],
				"aaaActionImpl", true));
		assertEquals("/aaa/bbb", QualifiedRule.getActionPath(new String[0],
				"aaa.bbbActionImpl", true));
		assertEquals("/ccc", QualifiedRule.getActionPath(
				new String[] { "aaa" }, "aaa.cccActionImpl", true));
		assertEquals("/bbb/ccc", QualifiedRule.getActionPath(
				new String[] { "aaa" }, "aaa.bbb.cccActionImpl", true));
		assertEquals("/bbb/ccc", QualifiedRule.getActionPath(new String[] {
				"aaa", "bbb" }, "aaa.bbb.cccActionImpl", true));
		assertEquals("/ccc/ddd/eee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb1.web", "aaa.bbb2.web" },
				"aaa.bbb1.web.ccc.DddEeeAction", true));
		assertEquals("/ccc/ddd/eee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb2.web", "aaa.bbb1.web" },
				"aaa.bbb1.web.ccc.DddEeeAction", true));
		assertEquals("/ccc/ddd/eee", QualifiedRule.getActionPath(new String[] {
				"aaa.web", "aaa.bbb1.web" }, "aaa.bbb1.web.ccc.DddEeeAction",
				true));
		assertEquals("/ccc/ddd/eee", QualifiedRule.getActionPath(new String[] {
				"aaa.bbb1.web", "aaa.web" }, "aaa.bbb1.web.ccc.DddEeeAction",
				true));
	}

	public void testToUpperNameToPath() {
		assertNull(QualifiedRule.splitUpperName(null));
		assertEquals("/foo/bar/hoge", QualifiedRule
				.splitUpperName("/foo/bar/hoge"));
		assertEquals("/foo/bar/hoge", QualifiedRule
				.splitUpperName("/foo/BarHoge"));
		assertEquals("/foo/bar/hoge", QualifiedRule
				.splitUpperName("/fooBarHoge"));
		assertEquals("/foo/bar/hoge", QualifiedRule
				.splitUpperName("/FooBarHoge"));
		assertEquals("/foo/_/bar/hoge", QualifiedRule
				.splitUpperName("/Foo_BarHoge"));
		assertEquals("/foo/-/bar/hoge", QualifiedRule
				.splitUpperName("/Foo-BarHoge"));
	}
}
